/*
* file CcComponent.java
*
* Licensed Materials - Property of IBM
* Restricted Materials of IBM
* 
* com.ibm.rational.wvcm.stp.cc.CcComponent
*
* (C) Copyright IBM Corporation 2004, 2013.  All Rights Reserved. 
* Note to U.S. Government Users Restricted Rights:  Use, duplication or  
* disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
*/

package com.ibm.rational.wvcm.stp.cc;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import javax.wvcm.Component;
import javax.wvcm.Feedback;
import javax.wvcm.PropertyNameList.PropertyName;
import javax.wvcm.ResourceList;
import javax.wvcm.WvcmException;

/**
 * A proxy for a ClearCase UCM component.
 * <p>
 * A UCM component defines the set of files that will be captured in a baseline
 * of that component. A baseline is a version of a component, and records a
 * version of each element selected by the stream's configuration.
 * </p>
 * <p>
 * A <i>rootless</i> component has no root directory element. Rootless
 * components are typically used to aggregate other components. Baselines of
 * rootless components contain other baselines, rather than file versions.
 * </p>
 * <p>
 * NOTE: Not all WVCM properties and operations are supported in this release of
 * CM API. For a list of properties currently supported by a particular resource
 * type, use doGetPropertyNameList() on an instance of that type:
 * </p>
 * 
 * <pre>
 *     PropertyRequest supportedProps = myResource.doGetPropertyNameList();
 * </pre>
 */
public interface CcComponent extends Component, CcVobResource
{
    /**
     * <p>Create a new UCM component at the location identified by this proxy. The
     * location should be an object name selector specifying the component's name
     * and the repository (project VOB) in which to create it.
     * </p>
     * <p>Set the {@link #ROOT_DIRECTORY_ELEMENT} property to specify the new component's
     * root.  If no root directory element is set, a rootless component is created.
     * </p>
     * <p>This method fails if the root directory element is not a legal choice for
     * a component root.
     * </p>
     */
    public CcComponent doCreateCcComponent(Feedback feedback) throws WvcmException;

    /**
     * Does this component have a root directory element?  Or is it "rootless"?
     */
    PropertyName<Boolean> HAS_ROOT_DIRECTORY_ELEMENT = 
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "has-root-directory-element");

    /**
     * Get the {@link #HAS_ROOT_DIRECTORY_ELEMENT} property of this component.
     * @return   true if this component has a root directory element;
     *           false if this is a rootless component.
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    boolean getHasRootDirectoryElement() throws WvcmException;

    /**
     * This component's initial baseline.  A component's initial
     * baseline contains exactly one version - the /main/0 version of the
     * component's root directory element.
     */
    PropertyName<CcBaseline> INITIAL_BASELINE = 
        new PropertyName<CcBaseline>(PROPERTY_NAMESPACE, "initial-baseline");

    /**
     * Get the the {@link #INITIAL_BASELINE} property of this component.
     * @return   A proxy for this component's initial baseline
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    CcBaseline getInitialBaseline() throws WvcmException;

    /**
     * This component's root directory element - the directory element in the
     * VOB that defines the scope of files that are captured in this component's
     * baselines.
     */
    PropertyName<CcElement> ROOT_DIRECTORY_ELEMENT = 
        new PropertyName<CcElement>(PROPERTY_NAMESPACE, "root-directory-element");

    /**
     * Get the {@link #ROOT_DIRECTORY_ELEMENT} property of this component.
     * @return   A proxy for this component's root directory element, or null if this is a
     *           rootless component.
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    CcElement getRootDirectoryElement() throws WvcmException;

    /**
     * Set the value of this component's {@link #ROOT_DIRECTORY_ELEMENT} property.
     * This property can only be set at component creation time.
     * 
     * @param root  A proxy for this component's root directory element
     */
    void setRootDirectoryElement(CcElement root);
    
    /**
     * This component's full closure of baselines.
     */
    PropertyName<ResourceList<CcBaseline>> BASELINE_LIST_CLOSURE = 
        new PropertyName<ResourceList<CcBaseline>>(PROPERTY_NAMESPACE, "baseline-list-closure");
    
    /**
     * Get the value of the this proxy's {@link #BASELINE_LIST_CLOSURE} property
     * @return a list of client proxies for this component's baselines
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    ResourceList<CcBaseline> getBaselineListClosure() throws WvcmException;
   
}
